@@ -20,7 +20,7 @@ from enum import Enum
 
 from ._internal import common as common_internal
 
-# TODO these should all have proper wrapper classes
+# TODO: these should all have proper wrapper classes
 
 DFSchema = common_internal.DFSchema
 DataType = common_internal.DataType
@@ -33,20 +33,26 @@ SqlStatistics = common_internal.SqlStatistics
 SqlTable = common_internal.SqlTable
 SqlType = common_internal.SqlType
 SqlView = common_internal.SqlView
+TableType = common_internal.TableType
+TableSource = common_internal.TableSource
+Constraints = common_internal.Constraints
 
 __all__ = [
+    "Constraints",
     "DFSchema",
     "DataType",
     "DataTypeMap",
-    "RexType",
-    "PythonType",
-    "SqlType",
     "NullTreatment",
-    "SqlTable",
-    "SqlSchema",
-    "SqlView",
-    "SqlStatistics",
+    "PythonType",
+    "RexType",
     "SqlFunction",
+    "SqlSchema",
+    "SqlStatistics",
+    "SqlTable",
+    "SqlType",
+    "SqlView",
+    "TableSource",
+    "TableType",
 ]
 
 
