@@ -19,15 +19,17 @@
 from __future__ import annotations
 
 from typing import (
+    TYPE_CHECKING,
     Any,
-    Callable,
-    Optional,
     Protocol,
     runtime_checkable,
 )
 
 from datafusion._internal import DataFrame as DataFrameInternal
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
 
 def _validate_positive_int(value: Any, param_name: str) -> None:
     """Validate that a parameter is a positive integer.
@@ -144,9 +146,9 @@ class DataFrameHtmlFormatter:
         min_rows_display: int = 20,
         repr_rows: int = 10,
         enable_cell_expansion: bool = True,
-        custom_css: Optional[str] = None,
+        custom_css: str | None = None,
         show_truncation_message: bool = True,
-        style_provider: Optional[StyleProvider] = None,
+        style_provider: StyleProvider | None = None,
         use_shared_styles: bool = True,
     ) -> None:
         """Initialize the HTML formatter.
@@ -226,8 +228,8 @@ class DataFrameHtmlFormatter:
         # Registry for custom type formatters
         self._type_formatters: dict[type, CellFormatter] = {}
         # Custom cell builders
-        self._custom_cell_builder: Optional[Callable[[Any, int, int, str], str]] = None
-        self._custom_header_builder: Optional[Callable[[Any], str]] = None
+        self._custom_cell_builder: Callable[[Any, int, int, str], str] | None = None
+        self._custom_header_builder: Callable[[Any], str] | None = None
 
     def register_formatter(self, type_class: type, formatter: CellFormatter) -> None:
         """Register a custom formatter for a specific data type.
@@ -368,7 +370,7 @@ if (!document.getElementById('df-styles')) {{
             f"max-height: {self.max_height}px; overflow: auto; border: "
             '1px solid #ccc;">'
         )
-        html.append('<table style="border-collapse: collapse; min-width: 100%">')
+        html.append('<table style="border-collapse: collapse">')
         return html
 
     def _build_table_header(self, schema: Any) -> list[str]:
