@@ -20,7 +20,7 @@ import datafusion as dfn
 import pyarrow as pa
 import pyarrow.dataset as ds
 import pytest
-from datafusion import SessionContext, Table
+from datafusion import SessionContext, Table, udtf
 
 
 # Note we take in `database` as a variable even though we don't use
@@ -53,7 +53,7 @@ def create_dataset() -> Table:
         names=["a", "b"],
     )
     dataset = ds.dataset([batch])
-    return Table.from_dataset(dataset)
+    return Table(dataset)
 
 
 class CustomSchemaProvider(dfn.catalog.SchemaProvider):
@@ -164,6 +164,28 @@ def test_python_table_provider(ctx: SessionContext):
     assert schema.table_names() == {"table4"}
 
 
+def test_schema_register_table_with_pyarrow_dataset(ctx: SessionContext):
+    schema = ctx.catalog().schema()
+    batch = pa.RecordBatch.from_arrays(
+        [pa.array([1, 2, 3]), pa.array([4, 5, 6])],
+        names=["a", "b"],
+    )
+    dataset = ds.dataset([batch])
+    table_name = "pa_dataset"
+
+    try:
+        schema.register_table(table_name, dataset)
+        assert table_name in schema.table_names()
+
+        result = ctx.sql(f"SELECT a, b FROM {table_name}").collect()
+
+        assert len(result) == 1
+        assert result[0].column(0) == pa.array([1, 2, 3])
+        assert result[0].column(1) == pa.array([4, 5, 6])
+    finally:
+        schema.deregister_table(table_name)
+
+
 def test_in_end_to_end_python_providers(ctx: SessionContext):
     """Test registering all python providers and running a query against them."""
 
@@ -210,3 +232,19 @@ def test_in_end_to_end_python_providers(ctx: SessionContext):
             assert len(batches) == 1
             assert batches[0].column(0) == pa.array([1, 2, 3])
             assert batches[0].column(1) == pa.array([4, 5, 6])
+
+
+def test_register_python_function_as_udtf(ctx: SessionContext):
+    basic_table = Table(ctx.sql("SELECT 3 AS value"))
+
+    @udtf("my_table_function")
+    def my_table_function_udtf() -> Table:
+        return basic_table
+
+    ctx.register_udtf(my_table_function_udtf)
+
+    result = ctx.sql("SELECT * FROM my_table_function()").collect()
+    assert len(result) == 1
+    assert len(result[0]) == 1
+    assert len(result[0][0]) == 1
+    assert result[0][0][0].as_py() == 3
