@@ -21,7 +21,7 @@ import numpy as np
 import pyarrow as pa
 import pyarrow.dataset as ds
 import pytest
-from datafusion import col, udf
+from datafusion import SessionContext, col, udf
 from datafusion.object_store import Http
 from pyarrow.csv import write_csv
 
@@ -157,6 +157,9 @@ def test_register_parquet(ctx, tmp_path):
     assert result.to_pydict() == {"cnt": [100]}
 
 
+@pytest.mark.filterwarnings(
+    "ignore:using literals for table_partition_cols data types:DeprecationWarning"
+)
 @pytest.mark.parametrize(
     ("path_to_str", "legacy_data_type"), [(True, False), (False, False), (False, True)]
 )
@@ -181,20 +184,31 @@ def test_register_parquet_partitioned(ctx, tmp_path, path_to_str, legacy_data_ty
 
     partition_data_type = "string" if legacy_data_type else pa.string()
 
-    ctx.register_parquet(
-        "datapp",
-        dir_root,
-        table_partition_cols=[("grp", partition_data_type)],
-        parquet_pruning=True,
-        file_extension=".parquet",
-    )
+    if legacy_data_type:
+        with pytest.warns(DeprecationWarning):
+            ctx.register_parquet(
+                "datapp",
+                dir_root,
+                table_partition_cols=[("grp", partition_data_type)],
+                parquet_pruning=True,
+                file_extension=".parquet",
+            )
+    else:
+        ctx.register_parquet(
+            "datapp",
+            dir_root,
+            table_partition_cols=[("grp", partition_data_type)],
+            parquet_pruning=True,
+            file_extension=".parquet",
+        )
+
     assert ctx.catalog().schema().names() == {"datapp"}
 
     result = ctx.sql("SELECT grp, COUNT(*) AS cnt FROM datapp GROUP BY grp").collect()
     result = pa.Table.from_batches(result)
 
     rd = result.to_pydict()
-    assert dict(zip(rd["grp"], rd["cnt"])) == {"a": 3, "b": 1}
+    assert dict(zip(rd["grp"], rd["cnt"], strict=False)) == {"a": 3, "b": 1}
 
 
 @pytest.mark.parametrize("path_to_str", [True, False])
@@ -340,7 +354,10 @@ def test_execute(ctx, tmp_path):
         result_values.extend(pydict["cnt"])
 
     result_keys, result_values = (
-        list(t) for t in zip(*sorted(zip(result_keys, result_values)))
+        list(t)
+        for t in zip(
+            *sorted(zip(result_keys, result_values, strict=False)), strict=False
+        )
     )
 
     assert result_keys == [1, 2, 3, 11, 12]
@@ -467,7 +484,7 @@ def test_simple_select(ctx, tmp_path, arr):
     # In DF 43.0.0 we now default to having BinaryView and StringView
     # so the array that is saved to the parquet is slightly different
     # than the array read. Convert to values for comparison.
-    if isinstance(result, (pa.BinaryViewArray, pa.StringViewArray)):
+    if isinstance(result, pa.BinaryViewArray | pa.StringViewArray):
         arr = arr.tolist()
         result = result.tolist()
 
@@ -524,7 +541,7 @@ def test_register_listing_table(
     result = pa.Table.from_batches(result)
 
     rd = result.to_pydict()
-    assert dict(zip(rd["grp"], rd["count"])) == {"a": 5, "b": 2}
+    assert dict(zip(rd["grp"], rd["count"], strict=False)) == {"a": 5, "b": 2}
 
     result = ctx.sql(
         "SELECT grp, COUNT(*) AS count FROM my_table WHERE date='2020-10-05' GROUP BY grp"  # noqa: E501
@@ -532,4 +549,47 @@ def test_register_listing_table(
     result = pa.Table.from_batches(result)
 
     rd = result.to_pydict()
-    assert dict(zip(rd["grp"], rd["count"])) == {"a": 3, "b": 2}
+    assert dict(zip(rd["grp"], rd["count"], strict=False)) == {"a": 3, "b": 2}
+
+
+def test_parameterized_named_params(ctx, tmp_path) -> None:
+    path = helpers.write_parquet(tmp_path / "a.parquet", helpers.data())
+
+    df = ctx.read_parquet(path)
+    result = ctx.sql(
+        "SELECT COUNT(a) AS cnt, $lit_val as lit_val FROM $replaced_df",
+        lit_val=3,
+        replaced_df=df,
+    ).collect()
+    result = pa.Table.from_batches(result)
+    assert result.to_pydict() == {"cnt": [100], "lit_val": [3]}
+
+
+def test_parameterized_param_values(ctx: SessionContext) -> None:
+    # Test the parameters that should be handled by the parser rather
+    # than our manipulation of the query string by searching for tokens
+    batch = pa.RecordBatch.from_arrays(
+        [pa.array([1, 2, 3, 4])],
+        names=["a"],
+    )
+
+    ctx.register_record_batches("t", [[batch]])
+    result = ctx.sql("SELECT a FROM t WHERE a < $val", param_values={"val": 3})
+    assert result.to_pydict() == {"a": [1, 2]}
+
+
+def test_parameterized_mixed_query(ctx: SessionContext) -> None:
+    batch = pa.RecordBatch.from_arrays(
+        [pa.array([1, 2, 3, 4])],
+        names=["a"],
+    )
+    ctx.register_record_batches("t", [[batch]])
+    registered_df = ctx.table("t")
+
+    result = ctx.sql(
+        "SELECT $col_name FROM $df WHERE a < $val",
+        param_values={"val": 3},
+        df=registered_df,
+        col_name="a",
+    )
+    assert result.to_pydict() == {"a": [1, 2]}
