@@ -15,33 +15,30 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::any::Any;
+use std::sync::Arc;
+
+use async_trait::async_trait;
+use datafusion::arrow::datatypes::SchemaRef;
+use datafusion::arrow::pyarrow::PyArrowType;
 use datafusion::catalog::Session;
+use datafusion::datasource::{TableProvider, TableType};
+use datafusion::error::{DataFusionError, Result as DFResult};
+use datafusion::logical_expr::{Expr, TableProviderFilterPushDown};
+use datafusion::physical_plan::ExecutionPlan;
 use pyo3::exceptions::PyValueError;
 /// Implements a Datafusion TableProvider that delegates to a PyArrow Dataset
 /// This allows us to use PyArrow Datasets as Datafusion tables while pushing down projections and filters
 use pyo3::prelude::*;
 use pyo3::types::PyType;
 
-use std::any::Any;
-use std::sync::Arc;
-
-use async_trait::async_trait;
-
-use datafusion::arrow::datatypes::SchemaRef;
-use datafusion::arrow::pyarrow::PyArrowType;
-use datafusion::datasource::{TableProvider, TableType};
-use datafusion::error::{DataFusionError, Result as DFResult};
-use datafusion::logical_expr::Expr;
-use datafusion::logical_expr::TableProviderFilterPushDown;
-use datafusion::physical_plan::ExecutionPlan;
-
 use crate::dataset_exec::DatasetExec;
 use crate::pyarrow_filter_expression::PyArrowFilterExpression;
 
 // Wraps a pyarrow.dataset.Dataset class and implements a Datafusion TableProvider around it
 #[derive(Debug)]
 pub(crate) struct Dataset {
-    dataset: PyObject,
+    dataset: Py<PyAny>,
 }
 
 impl Dataset {
@@ -73,7 +70,7 @@ impl TableProvider for Dataset {
 
     /// Get a reference to the schema for this table
     fn schema(&self) -> SchemaRef {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let dataset = self.dataset.bind(py);
             // This can panic but since we checked that self.dataset is a pyarrow.dataset.Dataset it should never
             Arc::new(
@@ -107,7 +104,7 @@ impl TableProvider for Dataset {
         // The datasource should return *at least* this number of rows if available.
         _limit: Option<usize>,
     ) -> DFResult<Arc<dyn ExecutionPlan>> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let plan: Arc<dyn ExecutionPlan> = Arc::new(
                 DatasetExec::new(py, self.dataset.bind(py), projection.cloned(), filters)
                     .map_err(|err| DataFusionError::External(Box::new(err)))?,
