@@ -17,12 +17,15 @@
 
 use std::fmt::{self, Display, Formatter};
 
-use datafusion::logical_expr::{logical_plan::Repartition, Expr, Partitioning};
-use pyo3::{prelude::*, IntoPyObjectExt};
+use datafusion::logical_expr::logical_plan::Repartition;
+use datafusion::logical_expr::{Expr, Partitioning};
+use pyo3::prelude::*;
+use pyo3::IntoPyObjectExt;
 
-use crate::{errors::py_type_err, sql::logical::PyLogicalPlan};
-
-use super::{logical_node::LogicalNode, PyExpr};
+use super::logical_node::LogicalNode;
+use super::PyExpr;
+use crate::errors::py_type_err;
+use crate::sql::logical::PyLogicalPlan;
 
 #[pyclass(frozen, name = "Repartition", module = "datafusion.expr", subclass)]
 #[derive(Clone)]
