@@ -17,14 +17,13 @@
 
 use std::sync::Arc;
 
-use pyo3::prelude::*;
-
 use object_store::aws::{AmazonS3, AmazonS3Builder};
 use object_store::azure::{MicrosoftAzure, MicrosoftAzureBuilder};
 use object_store::gcp::{GoogleCloudStorage, GoogleCloudStorageBuilder};
 use object_store::http::{HttpBuilder, HttpStore};
 use object_store::local::LocalFileSystem;
 use pyo3::exceptions::PyValueError;
+use pyo3::prelude::*;
 use url::Url;
 
 #[derive(FromPyObject)]
@@ -179,13 +178,14 @@ pub struct PyAmazonS3Context {
 #[pymethods]
 impl PyAmazonS3Context {
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (bucket_name, region=None, access_key_id=None, secret_access_key=None, endpoint=None, allow_http=false, imdsv1_fallback=false))]
+    #[pyo3(signature = (bucket_name, region=None, access_key_id=None, secret_access_key=None, session_token=None, endpoint=None, allow_http=false, imdsv1_fallback=false))]
     #[new]
     fn new(
         bucket_name: String,
         region: Option<String>,
         access_key_id: Option<String>,
         secret_access_key: Option<String>,
+        session_token: Option<String>,
         endpoint: Option<String>,
         //retry_config: RetryConfig,
         allow_http: bool,
@@ -206,6 +206,10 @@ impl PyAmazonS3Context {
             builder = builder.with_secret_access_key(secret_access_key);
         };
 
+        if let Some(session_token) = session_token {
+            builder = builder.with_token(session_token);
+        }
+
         if let Some(endpoint) = endpoint {
             builder = builder.with_endpoint(endpoint);
         };
