@@ -15,18 +15,18 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use pyo3::{prelude::*, types::PyBytes};
+use datafusion_substrait::logical_plan::{consumer, producer};
+use datafusion_substrait::serializer;
+use datafusion_substrait::substrait::proto::Plan;
+use prost::Message;
+use pyo3::prelude::*;
+use pyo3::types::PyBytes;
 
 use crate::context::PySessionContext;
 use crate::errors::{py_datafusion_err, PyDataFusionError, PyDataFusionResult};
 use crate::sql::logical::PyLogicalPlan;
 use crate::utils::wait_for_future;
 
-use datafusion_substrait::logical_plan::{consumer, producer};
-use datafusion_substrait::serializer;
-use datafusion_substrait::substrait::proto::Plan;
-use prost::Message;
-
 #[pyclass(frozen, name = "Plan", module = "datafusion.substrait", subclass)]
 #[derive(Debug, Clone)]
 pub struct PyPlan {
@@ -35,7 +35,7 @@ pub struct PyPlan {
 
 #[pymethods]
 impl PyPlan {
-    fn encode(&self, py: Python) -> PyResult<PyObject> {
+    fn encode(&self, py: Python) -> PyResult<Py<PyAny>> {
         let mut proto_bytes = Vec::<u8>::new();
         self.plan
             .encode(&mut proto_bytes)
@@ -93,7 +93,7 @@ impl PySubstraitSerializer {
         sql: &str,
         ctx: PySessionContext,
         py: Python,
-    ) -> PyDataFusionResult<PyObject> {
+    ) -> PyDataFusionResult<Py<PyAny>> {
         let proto_bytes: Vec<u8> =
             wait_for_future(py, serializer::serialize_bytes(sql, &ctx.ctx))??;
         Ok(PyBytes::new(py, &proto_bytes).into())
