@@ -15,18 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use pyo3::prelude::*;
 use std::sync::Arc;
 
+use datafusion::catalog::{TableFunctionImpl, TableProvider};
+use datafusion::error::Result as DataFusionResult;
+use datafusion::logical_expr::Expr;
+use datafusion_ffi::udtf::{FFI_TableFunction, ForeignTableFunction};
+use pyo3::prelude::*;
+use pyo3::types::{PyCapsule, PyTuple};
+
 use crate::errors::{py_datafusion_err, to_datafusion_err};
 use crate::expr::PyExpr;
 use crate::table::PyTable;
 use crate::utils::validate_pycapsule;
-use datafusion::catalog::{TableFunctionImpl, TableProvider};
-use datafusion::error::Result as DataFusionResult;
-use datafusion::logical_expr::Expr;
-use datafusion_ffi::udtf::{FFI_TableFunction, ForeignTableFunction};
-use pyo3::types::{PyCapsule, PyTuple};
 
 /// Represents a user defined table function
 #[pyclass(frozen, name = "TableFunction", module = "datafusion")]
@@ -39,7 +40,7 @@ pub struct PyTableFunction {
 // TODO: Implement pure python based user defined table functions
 #[derive(Debug, Clone)]
 pub(crate) enum PyTableFunctionInner {
-    PythonFunction(Arc<PyObject>),
+    PythonFunction(Arc<Py<PyAny>>),
     FFIFunction(Arc<dyn TableFunctionImpl>),
 }
 
@@ -83,7 +84,7 @@ impl PyTableFunction {
 
 #[allow(clippy::result_large_err)]
 fn call_python_table_function(
-    func: &Arc<PyObject>,
+    func: &Arc<Py<PyAny>>,
     args: &[Expr],
 ) -> DataFusionResult<Arc<dyn TableProvider>> {
     let args = args
@@ -92,7 +93,7 @@ fn call_python_table_function(
         .collect::<Vec<_>>();
 
     // move |args: &[ArrayRef]| -> Result<ArrayRef, DataFusionError> {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let py_args = PyTuple::new(py, args)?;
         let provider_obj = func.call1(py, py_args)?;
         let provider = provider_obj.bind(py);
